package cn.hhj.microboot.config;

import org.springframework.beans.factory.annotation.Value;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.data.redis.connection.RedisConnectionFactory;
import org.springframework.data.redis.connection.jedis.JedisConnectionFactory;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.data.redis.serializer.StringRedisSerializer;

import cn.hhj.microboot.util.redis.RedisObjectSerializer;
import redis.clients.jedis.JedisPoolConfig;

@Configuration
public class RedisTwoConfig {
	@Value("${spring.redisTwo.host}")
	private String hostName="192.168.136.132";
	@Value("${spring.redisTwo.port}")
	private int port=6379;
	@Value("${spring.redisTwo.pool.max-active}")
	private int maxActive=1000;
	@Value("${spring.redisTwo.pool.max-idle}")
	private int maxIdle=2;
	@Value("${spring.redisTwo.pool.min-idle}") 
	private int  minIdle=8;
	@Value("${spring.redisTwo.pool.max-wait}")
	private long maxWait=100;
	@Value("${spring.redisTwo.database}")
	private int database=0;
	public RedisConnectionFactory getRedisConnectionFactory(String hostName,int port,int database, int maxActive, int maxIdle, int minIdle,
			long maxWait) { // 是负责建立Factory的连接工厂类
		JedisConnectionFactory jedisFactory = new JedisConnectionFactory();
		jedisFactory.setHostName(hostName);
		jedisFactory.setPort(port);
		jedisFactory.setDatabase(database);
		JedisPoolConfig poolConfig = new JedisPoolConfig(); // 进行连接池配置
		poolConfig.setMaxTotal(maxActive);
		poolConfig.setMaxIdle(maxIdle);
		poolConfig.setMinIdle(minIdle);
		poolConfig.setMaxWaitMillis(maxWait);
		jedisFactory.setPoolConfig(poolConfig);
		jedisFactory.afterPropertiesSet(); // 初始化连接池配置
		return jedisFactory;
	}
	@Bean("redisTwo")
	public RedisTemplate<String, Object> getRedisTemplate() {
		RedisConnectionFactory factory = 
				this.getRedisConnectionFactory(hostName, port,database, maxActive, maxIdle, minIdle, maxWait); // 建立Redis的连接
		RedisTemplate<String, Object> redisTemplate = new RedisTemplate<String, Object>();
		redisTemplate.setConnectionFactory(factory);
		redisTemplate.setKeySerializer(new StringRedisSerializer()); // key的序列化类型
		redisTemplate.setValueSerializer(new RedisObjectSerializer()); // value的序列化类型
		return redisTemplate;
	}
}
