package cn.hhj.microboot.realm;

import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

import javax.annotation.Resource;

import org.apache.shiro.SecurityUtils;
import org.apache.shiro.authc.AuthenticationException;
import org.apache.shiro.authc.AuthenticationInfo;
import org.apache.shiro.authc.AuthenticationToken;
import org.apache.shiro.authc.IncorrectCredentialsException;
import org.apache.shiro.authc.LockedAccountException;
import org.apache.shiro.authc.SimpleAuthenticationInfo;
import org.apache.shiro.authc.UnknownAccountException;
import org.apache.shiro.authz.AuthorizationInfo;
import org.apache.shiro.authz.SimpleAuthorizationInfo;
import org.apache.shiro.realm.AuthorizingRealm;
import org.apache.shiro.subject.PrincipalCollection;
import org.springframework.web.client.RestTemplate;

import cn.hhj.util.enctype.PasswordUtil;
import cn.hhj.vo.Member;

public class MemberRealm extends AuthorizingRealm {
	@Resource
	private RestTemplate restTemplate ;
    String GET_URL="http://localhost:8001/member/get?mid=";
	String AUTH_URL="http://localhost:8001/member/auth?mid=";
	@Override
	protected AuthenticationInfo doGetAuthenticationInfo(AuthenticationToken token) throws AuthenticationException {
		System.out.println("********** 1、用户登录认证：doGetAuthenticationInfo() **********");
		// 1、登录认证的方法需要先执行，需要用他来判断登录的用户信息是否合法
		String username =token.getPrincipal().toString() ;	// 取得用户名
		// 需要通过用户名取得用户的完整信息，利用业务层操作
		Member vo = this.restTemplate.postForObject(GET_URL+username, null, Member.class);
		if (vo == null) {
			throw new UnknownAccountException("用户名或密码输入错误！") ;
		}
		// 用户名如果存在了，那么就需要确定密码是否正确
		String password = PasswordUtil.getPassword(new String((char[]) token.getCredentials()));
		if (!password.equals(vo.getPassword())) { // 密码验证
			throw new IncorrectCredentialsException("密码输入错误！");
		}
		// 随后还需要考虑用户被锁定的问题
		if (vo.getLocked().equals(1)) { // 1表示非0，非0就是true
			 throw new LockedAccountException("账号被锁定！");
		}
		// 定义需要进行返回的操作数据信息项，返回的认证信息使用应该是密文
		SimpleAuthenticationInfo auth = new SimpleAuthenticationInfo(token.getPrincipal(), password, "memberRealm");
		// 在认证完成之后可以直接取得用户所需要的信息内容，保存在Session之中
		SecurityUtils.getSubject().getSession().setAttribute("name", vo.getName());
		return auth;
	}
	@Override
	protected AuthorizationInfo doGetAuthorizationInfo(PrincipalCollection principals) {
		System.out.println("********** 2、用户角色与权限：doGetAuthorizationInfo **********");
		String username = (String) principals.getPrimaryPrincipal() ;	// 取得用户登录名
		SimpleAuthorizationInfo auth = new SimpleAuthorizationInfo() ;	// 定义授权信息的返回数据
		Map<String,Object> map =this.restTemplate.postForObject(AUTH_URL+username, null, Map.class);
		Set<String> allRoles=new HashSet<String>();
		Set<String> allActions=new HashSet<String>();
		allRoles.addAll((List<String>)map.get("allRoles"));
		allActions.addAll((List<String>)map.get("allActions"));
		auth.setRoles(allRoles);// 所有的角色必须以Set集合的形式出现
		auth.setStringPermissions(allActions); 	// 所有的权限必须以Set集合的形式出现
		return auth;
	} 
}
